package dev.peerat.parser.java;

import java.util.ArrayList;
import java.util.List;

import dev.peerat.parser.Token;

public class Class extends ClassBase{
	
	//where mod ????
	private Token name;
	private Token extend;
	private Token implement;
	
	private List<JavaElement> elements;
	
	public Class(List<Annotation> annotations, Token name){
		super(annotations);
		this.name = name;
		this.elements = new ArrayList<>();
	}
	
	public Class(List<Annotation> annotations, Token name, Token extend, Token implement){
		this(annotations, name);
		this.extend = extend;
		this.implement = implement;
	}

	@Override
	public void addFunction(Function function) {
		this.elements.add(function);
	}

	@Override
	public void addVariable(Variable variable) {
		this.elements.add(variable);
	}

	@Override
	public void addClass(ClassBase clazz){
		this.elements.add(clazz);
	}
	
	public Token getName(){
		return this.name;
	}
	
	public Token getExtension(){
		return this.extend;
	}
	
	public Token getImplementations(){
		return this.implement;
	}
	
	public List<JavaElement> getElements(){
		return this.elements;
	}

	@Override
	public void build(Builder builder) throws Exception{
		super.build(builder);
		
		builder.append("class ");
		builder.append(name);
		if(extend != null){
			builder.append(" extends ");
			builder.append(extend);
		}
		if(implement != null){
			builder.append(" implements ");
			builder.append(implement);
		}
		builder.append("{");
		if(elements != null){
			for(JavaElement element : this.elements){
				element.build(builder);
			}
		}
		builder.append("}");
	}

	@Override
	public <E extends JavaElement> E find(java.util.function.Function<JavaElement, Boolean> finder){
		E annotation = super.find(finder);
		if(annotation != null) return annotation;
		
		if(elements != null){
			for(JavaElement element : this.elements){
				if(finder.apply(element)) return (E) element;
			}
		}
		return null;
	}
	
	@Override
	public <E extends JavaElement> void findAll(java.util.function.Function<JavaElement, Boolean> finder, List<E> list){
		super.findAll(finder, list);
		if(elements != null){
			for(JavaElement element : this.elements){
				if(finder.apply(element)) list.add((E) element);
				element.findAll(finder, list);
			}
		}
	}
}
